.include "m8def.inc"

.def temp1 = r16
.def temp2 = r17
.def temp3 = r18
.def char = r19
.def charc = r20

;.equ CLOCK = 4000000				;Fr 4Mhz
.equ CLOCK = 3690000				;Fr STkK500
.equ BAUD = 9600
.equ UBRRVAL = CLOCK/(BAUD*16)-1

.org 0x00
	rjmp start

.org URXCaddr
	rjmp int_uart_recive			;Interrupt vektor fr UART-Empfang

start:								;Laden aller Eistellungen

	ldi temp1, LOW(RAMEND)			;Initalisieren des Stackpointers
	out SPL, temp1
	ldi temp1, HIGH(RAMEND)
	out SPH, temp1

	ldi temp1,0b11111111			;Port C Ausgang fr LCD
	out DDRC, temp1 

	ldi temp1,0b11111111			;Port C Ausgang fr LCD
	out DDRB, temp1 


	ldi temp1, LOW(UBRRVAL)			;Bautrate einstellen
	out UBRRL, temp1
	ldi temp1, HIGH(UBRRVAL)
	out UBRRH, temp1

	ldi temp1, (1<<URSEL)|(3<<UCSZ0)	;Frame-Format: 8 Data Bits, 1 Stop Bit
	out UCSRC, temp1

	ldi temp1, (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);Interupt bei Empfang,RX (Empfangen) einschalten,TX (Senden) einschalten=
	out UCSRB, temp1

	sei								;Interrupts einschalten

	rcall lcd_init					;Display initialisieren
	rcall lcd_clear					;Display lschen

	ldi ZL, LOW(text*2)				;Adresse des Strings in den
	ldi ZH, HIGH(text*2)			;Z-Pointer laden

	rcall print

	ldi charc, 5

main:
rjmp main							;Hauptprogramm (Endlosschleife)


int_uart_recive:					;Interrupt Rotine fr UART-Empfang
	push temp1						;Inhalt von temp1 aus dem Stack sichern

	in temp1, UDR					;Byte aus dem Empfangsregister laden

	cpi temp1, 0x7F					;Vergleichen ob DEL gesendet wurde
	breq recive_DEL

	cpi temp1, 0x03					;Vergleichen ob End of Text (ETX) gesendet wurde
	breq recive_ETX

	cpi temp1, 0x07					;Vergleichen ob Bell (BEL) gesendet wurde
	breq recive_BEL

	cpi temp1, 0x11					;Vergleichen ob Device Control 1 (DC1) gesendet wurde
	breq recive_DC1

	cpi temp1, 0x12					;Vergleichen ob Device Control 2 (DC2) gesendet wurde
	breq recive_DC2

	cpi temp1, 0x13					;Vergleichen ob Device Control 3 (DC3) gesendet wurde
	breq recive_DC3

	cpi temp1, 0x14					;Vergleichen ob Device Control 4 (DC4) gesendet wurde
	breq recive_DC4

	mov char, temp1

recive_back:						;Rcksprungspunkt
	pop temp1						;Inhalt von temp1 vom Stack wiederherstellen
	reti

;Funktion zum lesen aus dem Flash
print:
	lpm								;Erstes Byte des Strings nach R0 lesen
	tst R0							;R0 auf 0 testen
	breq print_end					;wenn 0, dann zu print_end
	mov temp1, r0					;Inhalt von R0 nach R16 kopieren
	rcall send_UART					;UART-Sendefunktion aufrufen
	rcall lcd_data					;LCD-Sendefunktion
	adiw ZL, 1						;Adresse des Z-Pointers um 1 erhhen
	rjmp print						;wieder zum Anfang springen
print_end:
	ret

send_UART:							;Unterprogramm zum senden ber UART
	sbis UCSRA, UDRE				;Prfen ob UART bereit ist zu senden
	rjmp send_UART					;Rcksprung zu send_UART
	out UDR, temp1					;Inhalt von temp1 ber UART senden
	ret								;Rcksprung

recive_DEL:
;	ldi temp1, 'D'
;	rcall send_UART					;UART-Sendefunktion aufrufen
	rcall lcd_clear					;LCD-Befehl zum Lschen des Displays

	ldi charc, 0

	rjmp recive_back

recive_ETX:
	mov temp1, char
;	rcall send_UART					;UART-Sendefunktion aufrufen
	rcall lcd_data					;LCD-Sendefunktion
	
	inc charc
	cpi charc, 16
	breq recive_DC2
	cpi charc, 32
	breq recive_DC3
	cpi charc, 48
	breq recive_DC4
	cpi charc, 64
	breq recive_DC1

	rjmp recive_back

recive_DC1:
;	ldi temp1, '1'
;	rcall send_UART					;UART-Sendefunktion aufrufen
	rcall lcd_home					;LCD-Befehl zum Rcksprung in die obere linke Ecke
	ldi charc, 0					;Zhler fr zeichen auf 0 setzen
	rjmp recive_back

recive_DC2:
;	ldi temp1, '2'
;	rcall send_UART					;UART-Sendefunktion aufrufen
	rcall lcd_goline2 				;LCD-Befehl zum Springen in die zweite Zeile
	ldi charc, 16					;Zhler fr zeichen auf 16 setzen
	rjmp recive_back

recive_DC3:
;	ldi temp1, '3'
;	rcall send_UART					;UART-Sendefunktion aufrufen
	rcall lcd_goline3				;LCD-Befehl zum Springen in die dritte Zeile
	ldi charc, 32					;Zhler fr zeichen auf 32 setzen
	rjmp recive_back

recive_DC4:
;	ldi temp1, '4'
;	rcall send_UART					;UART-Sendefunktion aufrufen
	rcall lcd_goline4				;LCD-Befehl zum Springen in die vierte Zeile
	ldi charc, 48					;Zhler fr zeichen auf 48 setzen
	rjmp recive_back

recive_BEL:
	ldi temp1, 7
	rcall send_UART
	rjmp recive_back

warten:                           ;Warteschleife 50 mS
	ldi temp2, 0
	ldi temp3, 0
schleife:
	inc temp2
	brne schleife
	inc temp3
	brne schleife
	ret


.include "lcd-routines.asm"			;LCD-Routinen werden hier eingefgt

text:
.db "TERM:", 0    ; Stringkonstante, durch eine 0 abgeschlossen